@@ -2,19 +2,19 @@ import pickle
 
 import pytest
 
-from multidict._compat import USE_CYTHON_EXTENSIONS
+from multidict._compat import USE_EXTENSIONS
 
 OPTIONAL_CYTHON = (
     ()
-    if USE_CYTHON_EXTENSIONS
-    else pytest.mark.skip(reason="No Cython extensions available")
+    if USE_EXTENSIONS
+    else pytest.mark.skip(reason="No extensions available")
 )
 
 
-@pytest.fixture(
+@pytest.fixture(  # type: ignore[call-overload]
     scope="session",
     params=[
-        pytest.param("multidict._multidict", marks=OPTIONAL_CYTHON),
+        pytest.param("multidict._multidict", marks=OPTIONAL_CYTHON),  # type: ignore
         "multidict._multidict_py",
     ],
 )
