@@ -1,11 +1,14 @@
 import pickle
 
-from multidict._compat import USE_CYTHON
+from multidict._compat import USE_EXTENSIONS
 from multidict._multidict_py import CIMultiDict as PyCIMultiDict  # noqa
 from multidict._multidict_py import MultiDict as PyMultiDict  # noqa
 
 try:
-    from multidict._multidict import CIMultiDict, MultiDict  # noqa
+    from multidict._multidict import (  # type: ignore # noqa
+        CIMultiDict,
+        MultiDict,
+    )
 except ImportError:
     pass
 
@@ -18,8 +21,8 @@ def write(name, proto):
 
 
 def generate():
-    if not USE_CYTHON:
-        raise RuntimeError("Cython is required")
+    if not USE_EXTENSIONS:
+        raise RuntimeError("C Extension is required")
     for proto in range(pickle.HIGHEST_PROTOCOL + 1):
         for name in ("MultiDict", "CIMultiDict", "PyMultiDict", "PyCIMultiDict"):
             write(name, proto)
