@@ -2,18 +2,18 @@ from collections import deque
 
 import pytest
 
-from multidict._compat import USE_CYTHON
+from multidict._compat import USE_EXTENSIONS
 from multidict._multidict_py import CIMultiDict as PyCIMultiDict
 from multidict._multidict_py import MultiDict as PyMultiDict  # noqa: E402
 
-if USE_CYTHON:
-    from multidict._multidict import CIMultiDict, MultiDict
+if USE_EXTENSIONS:
+    from multidict._multidict import CIMultiDict, MultiDict  # type: ignore
 
 
 @pytest.fixture(
-    params=([MultiDict, CIMultiDict] if USE_CYTHON else [])
+    params=([MultiDict, CIMultiDict] if USE_EXTENSIONS else [])
     + [PyMultiDict, PyCIMultiDict],
-    ids=(["MultiDict", "CIMultiDict"] if USE_CYTHON else [])
+    ids=(["MultiDict", "CIMultiDict"] if USE_EXTENSIONS else [])
     + ["PyMultiDict", "PyCIMultiDict"],
 )
 def cls(request):
